package org.example

import kotlin.test.Test
import kotlin.test.assertEquals

class ParserTest {

    @Test
    fun testSimpleCharacter() {
        val input = "a"
        val result = compileRegex(input)
        assertEquals("a", result.toString())
    }
    @Test
    fun testCharacterWithPlus() {
        val input = "a+"
        val result = compileRegex(input)
        assertEquals("a+", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("aa").isSuccess)
        assert(!result.match("b").isSuccess)
    }
    @Test
    fun testCharacterWithStar() {
        val input = "b*"
        val result = compileRegex(input)
        assertEquals("b*", result.toString())
        assert(result.match("").isSuccess)
        assert(result.match("b").isSuccess)
        assert(result.match("bb").isSuccess)
        assert(result.match("a").isSuccess)
    }
    @Test
    fun testCharacterWithQuestion() {
        val input = "c?"
        val result = compileRegex(input)
        assertEquals("c?", result.toString())
        assert(result.match("").isSuccess)
        assert(result.match("c").isSuccess)
        assert(result.match("cc").isSuccess)
    }
    @Test
    fun testDot() {
        val input = "."
        val result = compileRegex(input)
        assertEquals(".", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("1").isSuccess)
        assert(!result.match("").isSuccess)
    }

    @Test
    fun testAlternation() {
        val input = "a|b"
        val result = compileRegex(input)
        assertEquals("a|b", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("b").isSuccess)
        assert(!result.match("c").isSuccess)
    }
    @Test
    fun testParentheses() {
        val input = "(d)"
        val result = compileRegex(input)
        assertEquals("(d)", result.toString())
        assert(result.match("d").isSuccess)
        assert(!result.match("e").isSuccess)
    }

    @Test
    fun testComplexExpression() {
        val input = "a(b|c)*d+"
        val result = compileRegex(input)
        assertEquals("a(b|c)*d+", result.toString())
        assert(result.match("ad").isSuccess)
        assert(!result.match("ab").isSuccess)
        assert(result.match("acd").isSuccess)
        assert(result.match("abbbd").isSuccess)
        assert(!result.match("a").isSuccess)
        assert(!result.match("b").isSuccess)
    }
    @Test
    fun testAndThen() {
        val input = "ab"
        val result = compileRegex(input)
        assertEquals("ab", result.toString())
        assert(result.match("ab").isSuccess)
        assert(!result.match("a").isSuccess)
        assert(!result.match("b").isSuccess)
    }
    @Test
    fun testDotAndPlus() {
        val input = ".+a"
        val result = compileRegex(input)
        assertEquals(".+a", result.toString())
        assert(!result.match("a").isSuccess)
        assert(result.match("ba").isSuccess)
        assert(result.match("bca").isSuccess)
    }
    @Test
    fun testEscapedCharacter() {
        val input = "\\+"
        val result = compileRegex(input)
        assertEquals("\\+", result.toString())
        assert(result.match("+").isSuccess)
        assert(!result.match("a").isSuccess)
    }
    @Test
    fun testBracketContent() {
        val input = "[abc]"
        val result = compileRegex(input)
        assertEquals("[abc]", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("b").isSuccess)
        assert(result.match("c").isSuccess)
        assert(!result.match("d").isSuccess)
    }
    @Test
    fun testNestedGroups() {
        val input = "(a(b|c)d)+"
        val result = compileRegex(input)
        assertEquals("(a(b|c)d)+", result.toString())
        assert(!result.match("ad").isSuccess)
        assert(result.match("abd").isSuccess)
        assert(result.match("acd").isSuccess)
        assert(!result.match("a").isSuccess)
    }
    @Test
    fun testCaptureGroups() {
        val input = "(a)(b)"
        val result = compileRegex(input)
        assertEquals("(a)(b)", result.toString())
        val matchResult = result.match("ab")
        assert(matchResult.isSuccess)
        val captures = matchResult.available.first();
        assertEquals("a", captures.captures.get("0"))
        assertEquals("b", captures.captures.get("1"))
    }
}
